(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

open Printf

module Make(V:Constant.S) = struct
  include X86Base
  module V = V

  let reg_to_string r = match r with
  | EAX -> "%eax"
  | EBX -> "%ebx"
  | ECX -> "%ecx"
  | EDX -> "%edx"
  | ESI -> "%esi"
  | EDI -> "%edi" 
  | EBP -> "%ebp"
  | Internal i -> sprintf "i%i" i
  | _ -> assert false

  include
      ArchExtra.Make
      (struct
        module V = V 

	type arch_reg = reg
        let arch = Misc.X86
        let forbidden_regs = []
	let pp_reg = pp_reg
	let reg_compare = reg_compare
        let reg_to_string = reg_to_string
        let internal_init r =
          if reg_compare r loop_idx = 0 then Some "max_loop"
          else None
      end)

end
