(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

(* Copy files from litmus own files *)

open Globals

let cp_lib_file src dst =
  let _,in_chan = MyName.open_lib src in
  begin try MySys.cp in_chan dst 
  with e -> close_in in_chan  ; raise e end ;
  close_in in_chan

module Make(A:Arch.S)(Tar:Tar.S) =
  struct

    let actual_name name ext = Tar.outname (name ^ ext)

    let do_cpy fnames src tgt ext =
      let _,in_chan = MyName.open_lib (src ^ ext) in
      let fnames =
        begin try
          let fname = actual_name tgt ext in
          MySys.cp in_chan fname ;
          fname::fnames
        with e -> close_in in_chan  ; raise e end in
      close_in in_chan ;
      fnames

    let cpy fnames name ext = do_cpy fnames ("_" ^ name) name ext

    let affinity_base a = match get_target_os a with
    | Linux -> "_linux_affinity"
    | AIX -> "_aix_affinity"
    | os -> Warn.fatal "Affinity not implemented for %s" (pp_os os)

    let dump () =
      let fnames = cpy [] "utils" ".c" in
      let fnames = cpy fnames "utils" ".h" in
      let fnames = cpy fnames "outs" ".c" in
      let fnames = cpy fnames "outs" ".h" in
      let fnames =
        match get_affinity () with
        | AffinityNone -> fnames 
        | _ ->
            let affi = affinity_base A.arch in
            let fnames = do_cpy fnames affi "affinity" ".c" in
            let fnames = cpy fnames "affinity" ".h" in
            fnames in
      fnames
  end
